@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's sums of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2011 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "utils.h"
 #include "clifford.h"
 #include "ncmul.h"
+#include "compiler.h"
 
 #include <iostream>
 #include <limits>
@@ -259,6 +260,16 @@ bool add::info(unsigned inf) const
 	return inherited::info(inf);
 }
 
+bool add::is_polynomial(const ex & var) const
+{
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+		if (!(i->rest).is_polynomial(var)) {
+			return false;
+		}
+	}
+	return true;
+}
+
 int add::degree(const ex & s) const
 {
 	int deg = std::numeric_limits<int>::min();
@@ -297,7 +308,7 @@ ex add::coeff(const ex & s, int n) const
 {
 	std::auto_ptr<epvector> coeffseq(new epvector);
 	std::auto_ptr<epvector> coeffseq_cliff(new epvector);
-	char rl = clifford_max_label(s);
+	int rl = clifford_max_label(s);
 	bool do_clifford = (rl != -1);
 	bool nonscalar = false;
 
@@ -343,9 +354,6 @@ ex add::eval(int level) const
 	epvector::const_iterator i = seq.begin(), end = seq.end();
 	while (i != end) {
 		GINAC_ASSERT(!is_exactly_a<add>(i->rest));
-		if (is_exactly_a<numeric>(i->rest))
-			dbgprint();
-		GINAC_ASSERT(!is_exactly_a<numeric>(i->rest));
 		++i;
 	}
 #endif // def DO_GINAC_ASSERT
@@ -366,6 +374,33 @@ ex add::eval(int level) const
 	} else if (!overall_coeff.is_zero() && seq[0].rest.return_type() != return_types::commutative) {
 		throw (std::logic_error("add::eval(): sum of non-commutative objects has non-zero numeric term"));
 	}
+	
+	// if any terms in the sum still are purely numeric, then they are more
+	// appropriately collected into the overall coefficient
+	epvector::const_iterator last = seq.end();
+	epvector::const_iterator j = seq.begin();
+	int terms_to_collect = 0;
+	while (j != last) {
+		if (unlikely(is_a<numeric>(j->rest)))
+			++terms_to_collect;
+		++j;
+	}
+	if (terms_to_collect) {
+		std::auto_ptr<epvector> s(new epvector);
+		s->reserve(seq_size - terms_to_collect);
+		numeric oc = *_num1_p;
+		j = seq.begin();
+		while (j != last) {
+			if (unlikely(is_a<numeric>(j->rest)))
+				oc = oc.mul(ex_to<numeric>(j->rest)).mul(ex_to<numeric>(j->coeff));
+			else
+				s->push_back(*j);
+			++j;
+		}
+		return (new add(s, ex_to<numeric>(overall_coeff).add_dyn(oc)))
+		        ->setflag(status_flags::dynallocated);
+	}
+	
 	return this->hold();
 }
 
