@@ -3,7 +3,7 @@
  *  Implementation of sequences of expression pairs. */
 
 /*
- *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2011 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -371,23 +371,10 @@ ex expairseq::conjugate() const
 		return *this;
 	}
 	ex result = thisexpairseq(newepv ? *newepv : seq, x);
-	if (newepv) {
-		delete newepv;
-	}
+	delete newepv;
 	return result;
 }
 
-bool expairseq::is_polynomial(const ex & var) const
-{
-	if (!is_exactly_a<add>(*this) && !is_exactly_a<mul>(*this))
-		return basic::is_polynomial(var);
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
-		if (!(i->rest).is_polynomial(var))
-			return false;
-	}
-	return true;
-}
-
 bool expairseq::match(const ex & pattern, exmap & repl_lst) const
 {
 	// This differs from basic::match() because we want "a+b+c+d" to
