@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's products of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2010 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2011 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -347,6 +347,17 @@ bool mul::info(unsigned inf) const
 	return inherited::info(inf);
 }
 
+bool mul::is_polynomial(const ex & var) const
+{
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+		if (!i->rest.is_polynomial(var) ||
+		    (i->rest.has(var) && !i->coeff.info(info_flags::integer))) {
+			return false;
+		}
+	}
+	return true;
+}
+
 int mul::degree(const ex & s) const
 {
 	// Sum up degrees of factors
@@ -729,6 +740,9 @@ bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, exmap& repls,
 		int factor, int &nummatches, const std::vector<bool> &subsed,
 		std::vector<bool> &matched)
 {
+	GINAC_ASSERT(subsed.size() == e.nops());
+	GINAC_ASSERT(matched.size() == e.nops());
+
 	if (factor == (int)pat.nops())
 		return true;
 
@@ -760,8 +774,8 @@ bool mul::has(const ex & pattern, unsigned options) const
 	if(is_a<mul>(pattern)) {
 		exmap repls;
 		int nummatches = std::numeric_limits<int>::max();
-		std::vector<bool> subsed(seq.size(), false);
-		std::vector<bool> matched(seq.size(), false);
+		std::vector<bool> subsed(nops(), false);
+		std::vector<bool> matched(nops(), false);
 		if(algebraic_match_mul_with_mul(*this, pattern, repls, 0, nummatches,
 				subsed, matched))
 			return true;
@@ -771,8 +785,7 @@ bool mul::has(const ex & pattern, unsigned options) const
 
 ex mul::algebraic_subs_mul(const exmap & m, unsigned options) const
 {	
-	std::vector<bool> subsed(seq.size(), false);
-	exvector subsresult(seq.size());
+	std::vector<bool> subsed(nops(), false);
 	ex divide_by = 1;
 	ex multiply_by = 1;
 
@@ -781,7 +794,7 @@ ex mul::algebraic_subs_mul(const exmap & m, unsigned options) const
 		if (is_exactly_a<mul>(it->first)) {
 retry1:
 			int nummatches = std::numeric_limits<int>::max();
-			std::vector<bool> currsubsed(seq.size(), false);
+			std::vector<bool> currsubsed(nops(), false);
 			exmap repls;
 			
 			if(!algebraic_match_mul_with_mul(*this, it->first, repls, 0, nummatches, subsed, currsubsed))
@@ -829,6 +842,38 @@ retry1:
 	return ((*this)/divide_by)*multiply_by;
 }
 
+ex mul::conjugate() const
+{
+	// The base class' method is wrong here because we have to be careful at
+	// branch cuts. power::conjugate takes care of that already, so use it.
+	epvector *newepv = 0;
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+		if (newepv) {
+			newepv->push_back(split_ex_to_pair(recombine_pair_to_ex(*i).conjugate()));
+			continue;
+		}
+		ex x = recombine_pair_to_ex(*i);
+		ex c = x.conjugate();
+		if (c.is_equal(x)) {
+			continue;
+		}
+		newepv = new epvector;
+		newepv->reserve(seq.size());
+		for (epvector::const_iterator j=seq.begin(); j!=i; ++j) {
+			newepv->push_back(*j);
+		}
+		newepv->push_back(split_ex_to_pair(c));
+	}
+	ex x = overall_coeff.conjugate();
+	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
+		return *this;
+	}
+	ex result = thisexpairseq(newepv ? *newepv : seq, x);
+	delete newepv;
+	return result;
+}
+
+
 // protected
 
 /** Implementation of ex::diff() for a product.  It applies the product rule.
